@@ -1,6 +1,10 @@
 "use strict";
 
 
+var sloc0 = '' + location,
+    dbg_kbd = /[?&]dbgkbd\b/.exec(sloc0);
+
+
 // server state
 var server_md = dom_src.value;
 
@@ -92,7 +96,7 @@ var action_stack = null;
 var nlines = 0;
 var draw_md = (function () {
     var delay = 1;
-    function draw_md() {
+    var draw_md = function () {
         var t0 = Date.now();
         var src = dom_src.value;
         convert_markdown(src, dom_pre);
@@ -135,7 +139,7 @@ img_load.callbacks = [function () {
 
 // resize handler
 redraw = (function () {
-    function onresize() {
+    var onresize = function () {
         var y = (dom_hbar.offsetTop + dom_hbar.offsetHeight) + 'px';
         dom_wrap.style.top = y;
         dom_swrap.style.top = y;
@@ -143,12 +147,12 @@ redraw = (function () {
         map_src = genmap(dom_ref, map_src);
         map_pre = genmap(dom_pre, map_pre);
     }
-    function setsbs() {
+    var setsbs = function () {
         dom_wrap.className = '';
         dom_swrap.className = '';
         onresize();
     }
-    function modetoggle() {
+    var modetoggle = function () {
         var mode = dom_nsbs.innerHTML;
         dom_nsbs.innerHTML = mode == 'editor' ? 'preview' : 'editor';
         mode += ' single';
@@ -163,7 +167,7 @@ redraw = (function () {
     dom_sbs.onclick = setsbs;
     dom_nsbs.onclick = modetoggle;
 
-    onresize();
+    (IE ? modetoggle : onresize)();
     return onresize;
 })();
 
@@ -172,7 +176,7 @@ redraw = (function () {
 (function () {
     var skip_src = false, skip_pre = false;
 
-    function scroll(src, srcmap, dst, dstmap) {
+    var scroll = function (src, srcmap, dst, dstmap) {
         var y = src.scrollTop;
         if (y < 8) {
             dst.scrollTop = 0;
@@ -255,7 +259,7 @@ function Modpoll() {
         }
 
         console.log('modpoll...');
-        var url = (document.location + '').split('?')[0] + '?_=' + Date.now();
+        var url = (location + '').split('?')[0] + '?_=' + Date.now();
         var xhr = new XHR();
         xhr.open('GET', url, true);
         xhr.responseType = 'text';
@@ -278,6 +282,7 @@ function Modpoll() {
             return;
 
         var new_md = this.responseText,
+            new_mt = this.getResponseHeader('X-Lastmod3') || r.lastmod,
             server_ref = server_md.replace(/\r/g, ''),
             server_now = new_md.replace(/\r/g, '');
 
@@ -285,6 +290,7 @@ function Modpoll() {
         if (r.initial && server_ref != server_now)
             return modal.confirm('Your browser decided to show an outdated copy of the document!\n\nDo you want to load the latest version from the server instead?', function () {
                 dom_src.value = server_md = new_md;
+                last_modified = new_mt;
                 draw_md();
             }, null);
 
@@ -344,7 +350,7 @@ function save(e) {
         fd.append("lastmod", (force ? -1 : last_modified));
         fd.append("body", txt);
 
-        var url = (document.location + '').split('?')[0];
+        var url = (location + '').split('?')[0];
         var xhr = new XHR();
         xhr.open('POST', url, true);
         xhr.responseType = 'text';
@@ -366,14 +372,14 @@ function save(e) {
 
 function save_cb() {
     if (this.status !== 200)
-        return toast.err(0, 'Error!  The file was NOT saved.\n\n' + this.status + ": " + (this.responseText + '').replace(/^<pre>/, ""));
+        return toast.err(0, 'Error!  The file was NOT saved.\n\nError ' + this.status + ":\n" + unpre(this.responseText));
 
     var r;
     try {
         r = JSON.parse(this.responseText);
     }
     catch (ex) {
-        return toast.err(0, 'Failed to parse reply from server:\n\n' + this.responseText);
+        return toast.err(0, 'Error!  The file was likely NOT saved.\n\nFailed to parse reply from server:\n\n' + unpre(this.responseText));
     }
 
     if (!r.ok) {
@@ -402,7 +408,7 @@ function save_cb() {
 
 function run_savechk(lastmod, txt, btn, ntry) {
     // download the saved doc from the server and compare
-    var url = (document.location + '').split('?')[0] + '?_=' + Date.now();
+    var url = (location + '').split('?')[0] + '?_=' + Date.now();
     var xhr = new XHR();
     xhr.open('GET', url, true);
     xhr.responseType = 'text';
@@ -416,7 +422,7 @@ function run_savechk(lastmod, txt, btn, ntry) {
 
 function savechk_cb() {
     if (this.status !== 200)
-        return toast.err(0, 'Error!  The file was NOT saved.\n\n' + this.status + ": " + (this.responseText + '').replace(/^<pre>/, ""));
+        return toast.err(0, 'Error!  The file was NOT saved.\n\nError ' + this.status + ":\n" + unpre(this.responseText));
 
     var doc1 = this.txt.replace(/\r\n/g, "\n");
     var doc2 = this.responseText.replace(/\r\n/g, "\n");
@@ -448,7 +454,7 @@ function savechk_cb() {
 
 // firefox bug: initial selection offset isn't cleared properly through js
 var ff_clearsel = (function () {
-    if (navigator.userAgent.indexOf(') Gecko/') === -1)
+    if (UA.indexOf(') Gecko/') === -1)
         return function () { }
 
     return function () {
@@ -605,10 +611,10 @@ function md_newline() {
     var s = linebounds(true),
         ln = s.md.substring(s.n1, s.n2),
         m1 = /^( *)([0-9]+)(\. +)/.exec(ln),
-        m2 = /^[ \t>+-]*(\* )?/.exec(ln),
+        m2 = /^[ \t]*[>+*-]{0,2}[ \t]/.exec(ln),
         drop = dom_src.selectionEnd - dom_src.selectionStart;
 
-    var pre = m2[0];
+    var pre = m2 ? m2[0] : '';
     if (m1 !== null)
         pre = m1[1] + (parseInt(m1[2]) + 1) + m1[3];
 
@@ -695,8 +701,14 @@ function reLastIndexOf(txt, ptn, end) {
 // table formatter
 function fmt_table(e) {
     if (e) e.preventDefault();
-    //dom_tbox.className = '';
-
+    try {
+        fmt_table2();
+    }
+    catch (ex) {
+        return toast.err(7, 'table-format (CTRL-K) failed:\n' + ex);
+    }
+}
+function fmt_table2() {
     var txt = dom_src.value,
         ofs = dom_src.selectionStart,
         //o0 = txt.lastIndexOf('\n\n', ofs),
@@ -898,12 +910,12 @@ var set_lno = (function () {
         pv = null,
         lno = ebi('lno');
 
-    function poke() {
+    var poke = function () {
         clearTimeout(t);
         t = setTimeout(fire, 20);
     }
 
-    function fire() {
+    var fire = function () {
         try {
             clearTimeout(t);
 
@@ -928,77 +940,94 @@ var set_lno = (function () {
 
 // hotkeys / toolbar
 (function () {
-    function keydown(ev) {
-        ev = ev || window.event;
-        var kc = ev.code || ev.keyCode || ev.which;
-        //console.log(ev.key, ev.code, ev.keyCode, ev.which);
-        if (ctrl(ev) && (ev.code == "KeyS" || kc == 83)) {
+    var keydown = function (e) {
+        if (!e && window.event) {
+            e = window.event;
+            if (dev_fbw == 1) {
+                toast.warn(10, 'hello from fallback code ;_;\ncheck console trace');
+                console.error('using window.event');
+            }
+        }
+        var k = (e.key || e.code) + '',
+            editing = document.activeElement == dom_src;
+
+        if (k.startsWith('Key'))
+            k = k.slice(3);
+
+        var kl = k.toLowerCase();
+
+        if (dbg_kbd)
+            console.log('KBD', k, kl, e.key, e.code, e.keyCode, e.which);
+
+        if (ctrl(e) && kl == "s") {
             save();
             return false;
         }
-        if (ev.code == "Escape" || kc == 27) {
+        if (k == "Escape" || k == "Esc") {
             var d = ebi('helpclose');
             if (d)
                 d.click();
         }
-        if (document.activeElement != dom_src)
-            return true;
+        if (editing)
+            set_lno();
 
-        set_lno();
-
-        if (ctrl(ev)) {
-            if (ev.code == "KeyH" || kc == 72) {
-                md_header(ev.shiftKey);
+        if (ctrl(e)) {
+            if (kl == "e") {
+                dom_nsbs.click();
                 return false;
             }
-            if (ev.code == "KeyZ" || kc == 90) {
-                if (ev.shiftKey)
+            if (!editing)
+                return true;
+
+            if (kl == "h") {
+                md_header(e.shiftKey);
+                return false;
+            }
+            if (kl == "z") {
+                if (e.shiftKey)
                     action_stack.redo();
                 else
                     action_stack.undo();
 
                 return false;
             }
-            if (ev.code == "KeyY" || kc == 89) {
+            if (kl == "y") {
                 action_stack.redo();
                 return false;
             }
-            if (ev.code == "KeyK") {
+            if (kl == "k") {
                 fmt_table();
                 return false;
             }
-            if (ev.code == "KeyU") {
+            if (kl == "u") {
                 iter_uni();
                 return false;
             }
-            if (ev.code == "KeyE") {
-                dom_nsbs.click();
+            if (k == "ArrowUp" || k == "ArrowDown") {
+                md_p_jump(k == "ArrowDown");
                 return false;
             }
-            var up = ev.code == "ArrowUp" || kc == 38;
-            var dn = ev.code == "ArrowDown" || kc == 40;
-            if (up || dn) {
-                md_p_jump(dn);
-                return false;
-            }
-            if (ev.code == "KeyX" || ev.code == "KeyC") {
-                md_cut(ev.code == "KeyX");
+            if (kl == "x" || kl == "c") {
+                md_cut(kl == "x");
                 return true; //sic
             }
         }
         else {
-            if (ev.code == "Tab" || kc == 9) {
-                md_indent(ev.shiftKey);
+            if (!editing)
+                return true;
+
+            if (k == "Tab") {
+                md_indent(e.shiftKey);
                 return false;
             }
-            if (ev.code == "Home" || kc == 36) {
-                md_home(ev.shiftKey);
+            if (k == "Home") {
+                md_home(e.shiftKey);
                 return false;
             }
-            if (!ev.shiftKey && (ev.code == "Enter" || kc == 13)) {
+            if (!e.shiftKey && k.endsWith("Enter")) {
                 return md_newline();
             }
-            if (!ev.shiftKey && kc == 8) {
+            if (!e.shiftKey && k == "Backspace") {
                 return md_backspace();
             }
         }
@@ -1022,7 +1051,9 @@ ebi('help').onclick = function (e) {
     var dom = ebi('helpbox');
     var dtxt = dom.getElementsByTagName('textarea');
     if (dtxt.length > 0) {
-        convert_markdown(dtxt[0].value, dom);
+        var txt = dtxt[0].value;
+        if (!convert_markdown(txt, dom))
+            dom.innerText = txt.split('## markdown')[0];
         dom.innerHTML = '<a href="#" id="helpclose">close</a>' + dom.innerHTML;
     }
 
@@ -1050,7 +1081,7 @@ action_stack = (function () {
     var ignore = false;
     var ref = dom_src.value;
 
-    function diff(from, to, cpos) {
+    var diff = function (from, to, cpos) {
         if (from === to)
             return null;
 
@@ -1064,31 +1095,33 @@ action_stack = (function () {
         var p1 = from.length,
             p2 = to.length;
 
-        while (p1-- > 0 && p2-- > 0)
+        while (p1 --> 0 && p2 --> 0)
             if (from[p1] != to[p2])
                 break;
 
-        if (car > ++p1) {
+        if (car > ++p1)
             car = p1;
-        }
 
         var txt = from.substring(car, p1)
         return {
             car: car,
-            cdr: ++p2,
+            cdr: p2 + (car && 1),
             txt: txt,
             cpos: cpos
         };
     }
 
-    function undiff(from, change) {
+    var undiff = function (from, change) {
+        var t1 = from.substring(0, change.car),
+            t2 = from.substring(change.cdr);
+
         return {
-            txt: from.substring(0, change.car) + change.txt + from.substring(change.cdr),
+            txt: t1 + change.txt + t2,
             cpos: change.cpos
         };
     }
 
-    function apply(src, dst) {
+    var apply = function (src, dst) {
         dbg('undos(%d) redos(%d)', hist.un.length, hist.re.length);
 
         if (src.length === 0)
@@ -1112,7 +1145,7 @@ action_stack = (function () {
         return true;
     }
 
-    function schedule_push() {
+    var schedule_push = function () {
         if (ignore) {
             ignore = false;
             return;
@@ -1123,7 +1156,7 @@ action_stack = (function () {
         sched_timer = setTimeout(push, 500);
     }
 
-    function undo() {
+    var undo = function () {
         if (hist.re.length == 0) {
             clearTimeout(sched_timer);
             push();
@@ -1131,11 +1164,11 @@ action_stack = (function () {
         return apply(hist.un, hist.re);
     }
 
-    function redo() {
+    var redo = function () {
         return apply(hist.re, hist.un);
     }
 
-    function push() {
+    var push = function () {
         var newtxt = dom_src.value;
         var change = diff(ref, newtxt, sched_cpos);
         if (change !== null)
