@@ -2,19 +2,17 @@
 # coding: utf-8
 from __future__ import print_function, unicode_literals
 
-import os
 import json
+import os
 import shutil
 import tempfile
 import unittest
-from textwrap import dedent
 
+from copyparty import util
+from copyparty.authsrv import VFS, AuthSrv
 from tests import util as tu
 from tests.util import Cfg
 
-from copyparty.authsrv import AuthSrv, VFS
-from copyparty import util
-
 
 class TestVFS(unittest.TestCase):
     def setUp(self):
@@ -59,8 +57,10 @@ class TestVFS(unittest.TestCase):
         t2 = list(sorted(lst))
         self.assertEqual(t1, t2)
 
-    def test(self):
-        td = os.path.join(self.td, "vfs")
+    def wipe_vfs(self, td):
+        os.chdir("..")
+        if os.path.exists(td):
+            shutil.rmtree(td)
         os.mkdir(td)
         os.chdir(td)
 
@@ -74,7 +74,12 @@ class TestVFS(unittest.TestCase):
                         with open(fn, "w") as f:
                             f.write(fn)
 
+    def test(self):
+        td = os.path.join(self.td, "vfs")
+        self.wipe_vfs(td)
+
         # defaults
+        self.wipe_vfs(td)
         vfs = AuthSrv(Cfg(), self.log).vfs
         self.assertEqual(vfs.nodes, {})
         self.assertEqual(vfs.vpath, "")
@@ -83,6 +88,7 @@ class TestVFS(unittest.TestCase):
         self.assertAxs(vfs.axs.uwrite, ["*"])
 
         # single read-only rootfs (relative path)
+        self.wipe_vfs(td)
         vfs = AuthSrv(Cfg(v=["a/ab/::r"]), self.log).vfs
         self.assertEqual(vfs.nodes, {})
         self.assertEqual(vfs.vpath, "")
@@ -91,6 +97,7 @@ class TestVFS(unittest.TestCase):
         self.assertAxs(vfs.axs.uwrite, [])
 
         # single read-only rootfs (absolute path)
+        self.wipe_vfs(td)
         vfs = AuthSrv(Cfg(v=[td + "//a/ac/../aa//::r"]), self.log).vfs
         self.assertEqual(vfs.nodes, {})
         self.assertEqual(vfs.vpath, "")
@@ -99,6 +106,7 @@ class TestVFS(unittest.TestCase):
         self.assertAxs(vfs.axs.uwrite, [])
 
         # read-only rootfs with write-only subdirectory (read-write for k)
+        self.wipe_vfs(td)
         vfs = AuthSrv(
             Cfg(a=["k:k"], v=[".::r:rw,k", "a/ac/acb:a/ac/acb:w:rw,k"]),
             self.log,
@@ -163,6 +171,7 @@ class TestVFS(unittest.TestCase):
         self.assertEqual(list(virt), [])
 
         # admin-only rootfs with all-read-only subfolder
+        self.wipe_vfs(td)
         vfs = AuthSrv(
             Cfg(a=["k:k"], v=[".::rw,k", "a:a:r"]),
             self.log,
@@ -176,14 +185,18 @@ class TestVFS(unittest.TestCase):
         self.assertEqual(len(vfs.nodes), 1)
         self.assertEqual(n.vpath, "a")
         self.assertEqual(n.realpath, os.path.join(td, "a"))
-        self.assertAxs(n.axs.uread, ["*"])
+        self.assertAxs(n.axs.uread, ["*", "k"])
         self.assertAxs(n.axs.uwrite, [])
-        self.assertEqual(vfs.can_access("/", "*"), (False, False, False, False, False))
-        self.assertEqual(vfs.can_access("/", "k"), (True, True, False, False, False))
-        self.assertEqual(vfs.can_access("/a", "*"), (True, False, False, False, False))
-        self.assertEqual(vfs.can_access("/a", "k"), (True, False, False, False, False))
+        perm_na = (False, False, False, False, False, False, False, False)
+        perm_rw = (True, True, False, False, False, False, False, False)
+        perm_ro = (True, False, False, False, False, False, False, False)
+        self.assertEqual(vfs.can_access("/", "*"), perm_na)
+        self.assertEqual(vfs.can_access("/", "k"), perm_rw)
+        self.assertEqual(vfs.can_access("/a", "*"), perm_ro)
+        self.assertEqual(vfs.can_access("/a", "k"), perm_ro)
 
         # breadth-first construction
+        self.wipe_vfs(td)
         vfs = AuthSrv(
             Cfg(
                 v=[
@@ -206,6 +219,7 @@ class TestVFS(unittest.TestCase):
         self.undot(vfs, "./.././foo/..", "")
 
         # shadowing
+        self.wipe_vfs(td)
         vfs = AuthSrv(Cfg(v=[".::r", "b:a/ac:r"]), self.log).vfs
 
         fsp, r1, v1 = self.ls(vfs, "", "*")
@@ -230,11 +244,11 @@ class TestVFS(unittest.TestCase):
         cfg_path = os.path.join(self.td, "test.cfg")
         with open(cfg_path, "wb") as f:
             f.write(
-                dedent(
+                util.dedent(
                     """
                     u a:123
                     u asd:fgh:jkl
-                    
+
                     ./src
                     /dst
                     r a
@@ -243,6 +257,7 @@ class TestVFS(unittest.TestCase):
                 ).encode("utf-8")
             )
 
+        self.wipe_vfs(td)
         au = AuthSrv(Cfg(c=[cfg_path]), self.log)
         self.assertEqual(au.acct["a"], "123")
         self.assertEqual(au.acct["asd"], "fgh:jkl")
